local _, ns = ...
local WK = ns.WhoKnows
local _VERSION = tonumber(GetAddOnMetadata('WhoKnows', 'version'))
local print = WK.print
local debugPrint = WK.debugPrint
-------------------------------------------------------

-- initial settings
WK.db = {}
function WK.db:LoadSettings()
	local newLoad = false
	local player = WK.currentPlayer.name
	local realm = WK.currentPlayer.realm
	local faction = WK.currentPlayer.faction

	if WhoKnows_Data == nil then
		WhoKnows_Data = {}
		WhoKnows_Data.EnableRecipeCheck = true
		WhoKnows_Data.GarrisonBlueprintCheck = true
		WhoKnows_Data.version = _VERSION
	end
	if WhoKnows_Data.ScanEnableBag == nil then
		WhoKnows_Data.ScanEnableBag = true
		WhoKnows_Data.ScanEnableGuildBank = true
		WhoKnows_Data.ScanEnableBank = true
		WhoKnows_Data.ScanEnableLoot = true
		WhoKnows_Data.ScanEnableVendors = true
		WhoKnows_Data.ScanEnableMailbox = true
		WhoKnows_Data.ScanEnableAH = true
	end
	if not WhoKnows_Data.Skills then
		WhoKnows_Data.Skills = {}
	end
	if not WhoKnows_Data.Skills[realm] then
		WhoKnows_Data.Skills[realm] = {}
	end
	if not WhoKnows_Data.Skills[realm][faction] then
		WhoKnows_Data.Skills[realm][faction] = {}
	end
	if not WhoKnows_Data.Skills[realm][faction][player] then
		WhoKnows_Data.Skills[realm][faction][player] = {}
		WhoKnows_Data.Skills[realm][faction][player]["skillLvls"] = {}
		WhoKnows_Data.Skills[realm][faction][player]["skillsKnown"] = {}
		WhoKnows_Data.Skills[realm][faction][player]["level"] = WK.currentPlayer.level
		WhoKnows_Data.Skills[realm][faction][player]["class"] = WK.currentPlayer.class
		WhoKnows_Data.Skills[realm][faction][player]["excludeFromScan"] = false
		WK.currentPlayer.player = WK.currentPlayer:SetPlayer()
		newLoad = true
	end

	if not WhoKnows_Data.Colors then
		WhoKnows_Data.Colors = {}
		WhoKnows_Data.Colors.SelfLearnableColor = { 0.12, 1, 0, 1 } -- green
		WhoKnows_Data.Colors.AltLearnableColor = { 1, 1, 0, 1 } -- yellow
		WhoKnows_Data.Colors.AlreadyKnownColor = { 0.12, 0.39, 0.77, 1 } -- blue
		WhoKnows_Data.Colors.NotReadyToLearnColor = { 1, 0.5, 0, 1 } -- orange
		WhoKnows_Data.Colors.UnlearnableColor = { 0.768, 0.125, 0.231, 1 } -- red
	end
	WhoKnows_Data.version = WhoKnows_Data.version or _VERSION

	if not WhoKnows_Data.enableAddon then
		WhoKnows_Data.enableAddon = true
	end
	if not WhoKnows_Data.enableDebug then
		WhoKnows_Data.enableDebug = false
	end
	WK.debug = WhoKnows_Data.enableDebug;
	WK.currentPlayer.player = WK.currentPlayer:SetPlayer()
	if newLoad then
		local F = WK.wkframes:IntroDialog()
		F:Show()
	end
	if WhoKnows_Data.enableSelfLearnableColor == nil then
		WhoKnows_Data.enableSelfLearnableColor = true
		WhoKnows_Data.enableAltLearnableColor = true
		WhoKnows_Data.enableAlreadyKnownColor = true
		WhoKnows_Data.enableNotReadyToLearnColor = true
		WhoKnows_Data.enableUnlearnableColor = true
	end
	WhoKnows_Data.Skills[realm][faction][player]["level"] = WK.currentPlayer.level
	WhoKnows_Data.Skills[realm][faction][player]["class"] = WK.currentPlayer.class
end

function WK.db:UpdateDB()
	debugPrint("WK.db:UpdateDB()")
	local player = WK.currentPlayer.name
	local realm = WK.currentPlayer.realm
	local faction = WK.currentPlayer.faction
	local upgradePerformed = false

	if WhoKnows_Data.version <= 1.02 then
		if WhoKnows_Data.Skills == nil then
			local skills = {}
			skills = WK:deepcopy(WhoKnows_Data)
			for k, v in pairs(WhoKnows_Data) do
				if type(v) == "table" then
					skills[k] = WK:deepcopy(WhoKnows_Data[k])
					WhoKnows_Data[k] = nil
				end
			end
			WhoKnows_Data.Skills = WK:deepcopy(skills)
		end
		for _, realms in pairs(WhoKnows_Data.Skills) do
			for _, toon in pairs(realms) do
				if (toon["skillsKnown"]["Fishing"]) then
					toon["skillsKnown"]["Fishing"] = nil
				end
				if (toon["skillsKnown"]["Archaeology"]) then
					toon["skillsKnown"]["Archaeology"] = nil
				end
			end
		end
		WhoKnows_Data.version = 1.53
		upgradePerformed = true
	end
	--v1.55--
	if WhoKnows_Data.version <= 1.53 then
		WhoKnows_Data.itemGlow = nil
		WhoKnows_Data.ScanEnableBag = true;
		WhoKnows_Data.ScanEnableGuildBank = true;
		WhoKnows_Data.ScanEnableBank = true;
		WhoKnows_Data.ScanEnableLoot = true;
		WhoKnows_Data.ScanEnableVendors = true;
		WhoKnows_Data.version = 1.61
		print("WhoKnows upgraded to version " .. WhoKnows_Data.version)
		WK.IDFrame = WK.wkframes:IntroDialog()
		local t = [["\n\nUpdates:\n\n -Added GUI configuration options:\n     You can now set each color to any one you want.\
-Fixed a bug regarding toons an different realms (Thanks Peppyval :) )\
-Added factions so you only see reports on toons in the same faction as your current toon.\
-Started work on tracking Glyphs.\
-Please report any issues at |cff69CCF0http://wow.curseforge.com/addons/who-knows/tickets|r"]]
		WK.IDFrame.body:SetWidth(638)
		WK.IDFrame:SetWidth(650)
		WK.IDFrame.body:SetText(t)
		WK.IDFrame:Show()
		upgradePerformed = true
	end
	if WhoKnows_Data.version < 1.62 then
		-- check for old style on current toon and move over
		if WhoKnows_Data.Skills[realm][player] then
			if not WhoKnows_Data.Skills[realm][faction] then WhoKnows_Data.Skills[realm][faction] = {} end
			WhoKnows_Data.Skills[realm][faction][player] = WhoKnows_Data.Skills[realm][player]
			WhoKnows_Data.Skills[realm][player] = nil
		end
		local hasUpgraded = true -- ugh... there's gotta be a better way to do this
		for _, toons in pairs(WhoKnows_Data.Skills) do
			for k in pairs(toons) do
				hasUpgraded = (k == "Horde" or k == "Alliance")
				if not hasUpgraded then break end
			end
			if not hasUpgraded then break end
		end
		WhoKnows_Data.EnableRecipeCheck = true
		if hasUpgraded then
			WhoKnows_Data.version = 1.62
			upgradePerformed = true
		else
			WK.wkframes:AssignFaction()
		end
	end

	if WhoKnows_Data.version < 2.55 then
		for realm_ in pairs(WhoKnows_Data.Skills) do
			for faction_ in pairs(WhoKnows_Data.Skills[realm]) do
				for player_ in pairs(WhoKnows_Data.Skills[realm][faction]) do
					WhoKnows_Data.Skills[realm_][faction_][player_]["excludeFromScan"] = false
					WhoKnows_Data.Skills[realm_][faction_][player_]["glyphs"]["dark succor"] = nil
					WhoKnows_Data.Skills[realm_][faction_][player_]["glyphs"]["afterlife"] = nil
					WhoKnows_Data.Skills[realm_][faction_][player_]["glyphs"]["healing storm"] = nil
				end
			end
		end
		WhoKnows_Data.itemGlow = nil
		WhoKnows_Data.ScanEnableMailbox = true
		WhoKnows_Data.EnablePetCheck = nil
		WhoKnows_Data.GarrisonBlueprintCheck = true
		WhoKnows_Data.version = 2.55
		upgradePerformed = true
	end

	if WhoKnows_Data.version < 3.21 then
		--remove glyphs :
		for realm_ in pairs(WhoKnows_Data.Skills) do
			for faction_ in pairs(WhoKnows_Data.Skills[realm]) do
				for player_ in pairs(WhoKnows_Data.Skills[realm][faction]) do
					WhoKnows_Data.Skills[realm_][faction_][player_]["glyphs"] = nil
				end
			end
		end
		upgradePerformed = true
	end
	if WhoKnows_Data.version < 3.71 then WhoKnows_Data.version = 3.71 end
	if WhoKnows_Data.version < 3.75 then
		debugPrint("Starting upgrade to 3.75")
		--TODO: make frame describing that toons need to be reloaded
		local msg = [[The way that Who Knows tracks known abilities was completely rewritten.
Any known ability for all of your toons will be reset to use the new method.
Once you open the Professions interface and view each professions skills Who Knows will once again have the required data.


Please report bugs or problems at http://wow.curseforge.com/addons/who-knows/tickets]]
		WK.wkframes:IntroDialog(msg):Show()
		for realm_ in pairs(WhoKnows_Data.Skills) do
			for faction_ in pairs(WhoKnows_Data.Skills[realm_]) do
				for player_ in pairs(WhoKnows_Data.Skills[realm_][faction_]) do
					table.wipe(WhoKnows_Data.Skills[realm_][faction_][player_]["skillLvls"])
					table.wipe(WhoKnows_Data.Skills[realm_][faction_][player_]["skillsKnown"])
					local cFaction = (faction_ == "Horde") and "|cffc41f3bHorde|r" or "|cff0070deAlliance|r"
					print(string.format("%s > %s > |cfffbcb02%s|r reset", realm_, cFaction, player_))
				end
			end
		end
		debugPrint("Setting version to 3.75")
		 WhoKnows_Data.version = 3.75
		upgradePerformed = true
	end
	if upgradePerformed == true then
		print("WhoKnows upgraded to version " .. WhoKnows_Data.version)
	end
end
